@@ -18,8 +18,10 @@
 #include <opal-api.h>
 #include <pci.h>
 #include <pci-cfg.h>
+#include <pci-slot.h>
+#include <opal-msg.h>
 #include <timebase.h>
-#include <lock.h>
+#include <timer.h>
 
 #define OPAL_PCICFG_ACCESS(op, cb, type)	\
 static int64_t opal_pci_config_##op(uint64_t phb_id,			\
@@ -31,10 +33,9 @@ static int64_t opal_pci_config_##op(uint64_t phb_id,			\
 									\
 	if (!phb)							\
 		return OPAL_PARAMETER;					\
-	phb->ops->lock(phb);						\
+	phb_lock(phb);						\
 	rc = phb->ops->cfg_##cb(phb, bus_dev_func, offset, data);	\
-	phb->ops->unlock(phb);						\
-	pci_put_phb(phb);						\
+	phb_unlock(phb);						\
 									\
 	return rc;							\
 }
@@ -85,11 +86,10 @@ static int64_t opal_pci_eeh_freeze_status(uint64_t phb_id, uint64_t pe_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->eeh_freeze_status)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->eeh_freeze_status(phb, pe_number, freeze_state,
 					 pci_error_type, NULL, phb_status);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -105,10 +105,9 @@ static int64_t opal_pci_eeh_freeze_clear(uint64_t phb_id, uint64_t pe_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->eeh_freeze_clear)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->eeh_freeze_clear(phb, pe_number, eeh_action_token);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -124,10 +123,9 @@ static int64_t opal_pci_eeh_freeze_set(uint64_t phb_id, uint64_t pe_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->eeh_freeze_set)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->eeh_freeze_set(phb, pe_number, eeh_action_token);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -149,10 +147,9 @@ static int64_t opal_pci_err_inject(uint64_t phb_id, uint32_t pe_no,
 	    type != OPAL_ERR_INJECT_TYPE_IOA_BUS_ERR64)
 		return OPAL_PARAMETER;
 
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->err_inject(phb, pe_no, type, func, addr, mask);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -168,10 +165,9 @@ static int64_t opal_pci_phb_mmio_enable(uint64_t phb_id, uint16_t window_type,
 		return OPAL_PARAMETER;
 	if (!phb->ops->phb_mmio_enable)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->phb_mmio_enable(phb, window_type, window_num, enable);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -191,11 +187,10 @@ static int64_t opal_pci_set_phb_mem_window(uint64_t phb_id,
 		return OPAL_PARAMETER;
 	if (!phb->ops->set_phb_mem_window)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->set_phb_mem_window(phb, window_type, window_num,
 					  addr, pci_addr, size);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -213,11 +208,10 @@ static int64_t opal_pci_map_pe_mmio_window(uint64_t phb_id, uint16_t pe_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->map_pe_mmio_window)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->map_pe_mmio_window(phb, pe_number, window_type,
 					  window_num, segment_num);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -247,11 +241,10 @@ static int64_t opal_pci_set_pe(uint64_t phb_id, uint64_t pe_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->set_pe)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->set_pe(phb, pe_number, bus_dev_func, bus_compare,
 			      dev_compare, func_compare, pe_action);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -267,10 +260,9 @@ static int64_t opal_pci_set_peltv(uint64_t phb_id, uint32_t parent_pe,
 		return OPAL_PARAMETER;
 	if (!phb->ops->set_peltv)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->set_peltv(phb, parent_pe, child_pe, state);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -286,10 +278,9 @@ static int64_t opal_pci_set_mve(uint64_t phb_id, uint32_t mve_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->set_mve)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->set_mve(phb, mve_number, pe_number);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -305,10 +296,9 @@ static int64_t opal_pci_set_mve_enable(uint64_t phb_id, uint32_t mve_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->set_mve_enable)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->set_mve_enable(phb, mve_number, state);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -344,15 +334,35 @@ static int64_t opal_pci_msi_eoi(uint64_t phb_id,
 		return OPAL_PARAMETER;
 	if (!phb->ops->pci_msi_eoi)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->pci_msi_eoi(phb, hwirq);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
 opal_call(OPAL_PCI_MSI_EOI, opal_pci_msi_eoi, 2);
 
+static int64_t opal_pci_tce_kill(uint64_t phb_id,
+				 uint32_t kill_type,
+				 uint32_t pe_num, uint32_t tce_size,
+				 uint64_t dma_addr, uint32_t npages)
+{
+	struct phb *phb = pci_get_phb(phb_id);
+	int64_t rc;
+
+	if (!phb)
+		return OPAL_PARAMETER;
+	if (!phb->ops->tce_kill)
+		return OPAL_UNSUPPORTED;
+	phb_lock(phb);
+	rc = phb->ops->tce_kill(phb, kill_type, pe_num, tce_size,
+				dma_addr, npages);
+	phb_unlock(phb);
+
+	return rc;
+}
+opal_call(OPAL_PCI_TCE_KILL, opal_pci_tce_kill, 6);
+
 static int64_t opal_pci_set_xive_pe(uint64_t phb_id, uint32_t pe_number,
 				    uint32_t xive_num)
 {
@@ -363,10 +373,9 @@ static int64_t opal_pci_set_xive_pe(uint64_t phb_id, uint32_t pe_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->set_xive_pe)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->set_xive_pe(phb, pe_number, xive_num);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -382,10 +391,9 @@ static int64_t opal_get_xive_source(uint64_t phb_id, uint32_t xive_num,
 		return OPAL_PARAMETER;
 	if (!phb->ops->get_xive_source)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->get_xive_source(phb, xive_num, interrupt_source_number);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -402,11 +410,10 @@ static int64_t opal_get_msi_32(uint64_t phb_id, uint32_t mve_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->get_msi_32)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->get_msi_32(phb, mve_number, xive_num, msi_range,
 				  msi_address, message_data);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -423,11 +430,10 @@ static int64_t opal_get_msi_64(uint64_t phb_id, uint32_t mve_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->get_msi_64)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->get_msi_64(phb, mve_number, xive_num, msi_range,
 				  msi_address, message_data);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -447,12 +453,11 @@ static int64_t opal_pci_map_pe_dma_window(uint64_t phb_id, uint16_t pe_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->map_pe_dma_window)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->map_pe_dma_window(phb, pe_number, window_id,
 					 tce_levels, tce_table_addr,
 					 tce_table_size, tce_page_size);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -471,46 +476,48 @@ static int64_t opal_pci_map_pe_dma_window_real(uint64_t phb_id,
 		return OPAL_PARAMETER;
 	if (!phb->ops->map_pe_dma_window_real)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->map_pe_dma_window_real(phb, pe_number, window_id,
 					      pci_start_addr, pci_mem_size);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
 opal_call(OPAL_PCI_MAP_PE_DMA_WINDOW_REAL, opal_pci_map_pe_dma_window_real, 5);
 
-static int64_t opal_pci_reset(uint64_t phb_id, uint8_t reset_scope,
+static int64_t opal_pci_reset(uint64_t id, uint8_t reset_scope,
                               uint8_t assert_state)
 {
-	struct phb *phb = pci_get_phb(phb_id);
+	struct pci_slot *slot = pci_slot_find(id);
+	struct phb *phb = slot ? slot->phb : NULL;
 	int64_t rc = OPAL_SUCCESS;
 
-	if (!phb)
+	if (!slot || !phb)
 		return OPAL_PARAMETER;
-	if (!phb->ops)
-		return OPAL_UNSUPPORTED;
 	if (assert_state != OPAL_ASSERT_RESET &&
 	    assert_state != OPAL_DEASSERT_RESET)
 		return OPAL_PARAMETER;
 
-	phb->ops->lock(phb);
+	phb_lock(phb);
 
 	switch(reset_scope) {
 	case OPAL_RESET_PHB_COMPLETE:
-		if (!phb->ops->complete_reset) {
+		/* Complete reset is applicable to PHB slot only */
+		if (!slot->ops.creset || slot->pd) {
 			rc = OPAL_UNSUPPORTED;
 			break;
 		}
 
-		rc = phb->ops->complete_reset(phb, assert_state);
+		if (assert_state != OPAL_ASSERT_RESET)
+			break;
+
+		rc = slot->ops.creset(slot);
 		if (rc < 0)
-			prerror("PHB#%d: Failure on complete reset, rc=%lld\n",
-				phb->opal_id, rc);
+			prlog(PR_ERR, "SLOT-%016llx: Error %lld on complete reset\n",
+			      slot->id, rc);
 		break;
 	case OPAL_RESET_PCI_FUNDAMENTAL:
-		if (!phb->ops->fundamental_reset) {
+		if (!slot->ops.freset) {
 			rc = OPAL_UNSUPPORTED;
 			break;
 		}
@@ -519,13 +526,13 @@ static int64_t opal_pci_reset(uint64_t phb_id, uint8_t reset_scope,
 		if (assert_state != OPAL_ASSERT_RESET)
 			break;
 
-		rc = phb->ops->fundamental_reset(phb);
+		rc = slot->ops.freset(slot);
 		if (rc < 0)
-			prerror("PHB#%d: Failure on fundamental reset, rc=%lld\n",
-				phb->opal_id, rc);
+			prlog(PR_ERR, "SLOT-%016llx: Error %lld on fundamental reset\n",
+			      slot->id, rc);
 		break;
 	case OPAL_RESET_PCI_HOT:
-		if (!phb->ops->hot_reset) {
+		if (!slot->ops.hreset) {
 			rc = OPAL_UNSUPPORTED;
 			break;
 		}
@@ -534,28 +541,39 @@ static int64_t opal_pci_reset(uint64_t phb_id, uint8_t reset_scope,
 		if (assert_state != OPAL_ASSERT_RESET)
 			break;
 
-		rc = phb->ops->hot_reset(phb);
+		rc = slot->ops.hreset(slot);
 		if (rc < 0)
-			prerror("PHB#%d: Failure on hot reset, rc=%lld\n",
-				phb->opal_id, rc);
+			prlog(PR_ERR, "SLOT-%016llx: Error %lld on hot reset\n",
+			      slot->id, rc);
 		break;
 	case OPAL_RESET_PCI_IODA_TABLE:
+		/* It's allowed on PHB slot only */
+		if (slot->pd || !phb->ops || !phb->ops->ioda_reset) {
+			rc = OPAL_UNSUPPORTED;
+			break;
+		}
+
 		if (assert_state != OPAL_ASSERT_RESET)
 			break;
-		if (phb->ops->ioda_reset)
-			phb->ops->ioda_reset(phb, true);
+
+		rc = phb->ops->ioda_reset(phb, true);
 		break;
 	case OPAL_RESET_PHB_ERROR:
+		/* It's allowed on PHB slot only */
+		if (slot->pd || !phb->ops || !phb->ops->papr_errinjct_reset) {
+			rc = OPAL_UNSUPPORTED;
+			break;
+		}
+
 		if (assert_state != OPAL_ASSERT_RESET)
 			break;
-		if (phb->ops->papr_errinjct_reset)
-			phb->ops->papr_errinjct_reset(phb);
+
+		rc = phb->ops->papr_errinjct_reset(phb);
 		break;
 	default:
 		rc = OPAL_UNSUPPORTED;
 	}
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return (rc > 0) ? tb_to_msecs(rc) : rc;
 }
@@ -573,29 +591,28 @@ static int64_t opal_pci_reinit(uint64_t phb_id,
 	if (!phb->ops || !phb->ops->pci_reinit)
 		return OPAL_UNSUPPORTED;
 
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->pci_reinit(phb, reinit_scope, data);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
 opal_call(OPAL_PCI_REINIT, opal_pci_reinit, 3);
 
-static int64_t opal_pci_poll(uint64_t phb_id)
+static int64_t opal_pci_poll(uint64_t id)
 {
-	struct phb *phb = pci_get_phb(phb_id);
+	struct pci_slot *slot = pci_slot_find(id);
+	struct phb *phb = slot ? slot->phb : NULL;
 	int64_t rc;
 
-	if (!phb)
+	if (!slot || !phb)
 		return OPAL_PARAMETER;
-	if (!phb->ops || !phb->ops->poll)
+	if (!slot->ops.poll)
 		return OPAL_UNSUPPORTED;
 
-	phb->ops->lock(phb);
-	rc = phb->ops->poll(phb);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_lock(phb);
+	rc = slot->ops.poll(slot);
+	phb_unlock(phb);
 
 	/* Return milliseconds for caller to sleep: round up */
 	if (rc > 0) {
@@ -608,6 +625,170 @@ static int64_t opal_pci_poll(uint64_t phb_id)
 }
 opal_call(OPAL_PCI_POLL, opal_pci_poll, 1);
 
+static int64_t opal_pci_get_presence_state(uint64_t id, uint64_t data)
+{
+	struct pci_slot *slot = pci_slot_find(id);
+	struct phb *phb = slot ? slot->phb : NULL;
+	uint8_t *presence = (uint8_t *)data;
+	int64_t rc;
+
+	if (!slot || !phb)
+		return OPAL_PARAMETER;
+	if (!slot->ops.get_presence_state)
+		return OPAL_UNSUPPORTED;
+
+	phb_lock(phb);
+	rc = slot->ops.get_presence_state(slot, presence);
+	phb_unlock(phb);
+
+	return rc;
+}
+opal_call(OPAL_PCI_GET_PRESENCE_STATE, opal_pci_get_presence_state, 2);
+
+static int64_t opal_pci_get_power_state(uint64_t id, uint64_t data)
+{
+	struct pci_slot *slot = pci_slot_find(id);
+	struct phb *phb = slot ? slot->phb : NULL;
+	uint8_t *power_state = (uint8_t *)data;
+	int64_t rc;
+
+	if (!slot || !phb)
+		return OPAL_PARAMETER;
+	if (!slot->ops.get_power_state)
+		return OPAL_UNSUPPORTED;
+
+	phb_lock(phb);
+	rc = slot->ops.get_power_state(slot, power_state);
+	phb_unlock(phb);
+
+	return rc;
+}
+opal_call(OPAL_PCI_GET_POWER_STATE, opal_pci_get_power_state, 2);
+
+static void set_power_timer(struct timer *t __unused, void *data,
+			    uint64_t now __unused)
+{
+	struct pci_slot *slot = data;
+	struct phb *phb = slot->phb;
+	struct pci_device *pd = slot->pd;
+	struct dt_node *dn = pd->dn;
+	uint8_t link;
+
+	switch (slot->state) {
+	case PCI_SLOT_STATE_SPOWER_START:
+		if (slot->retries-- == 0) {
+			pci_slot_set_state(slot, PCI_SLOT_STATE_NORMAL);
+			opal_queue_msg(OPAL_MSG_ASYNC_COMP, NULL, NULL,
+				       slot->async_token, dn->phandle,
+				       slot->power_state, OPAL_BUSY);
+		} else {
+			schedule_timer(&slot->timer, msecs_to_tb(10));
+		}
+
+		break;
+	case PCI_SLOT_STATE_SPOWER_DONE:
+		if (slot->power_state == OPAL_PCI_SLOT_POWER_OFF) {
+			pci_remove_bus(phb, &pd->children);
+			pci_slot_set_state(slot, PCI_SLOT_STATE_NORMAL);
+			opal_queue_msg(OPAL_MSG_ASYNC_COMP, NULL, NULL,
+				       slot->async_token, dn->phandle,
+				       OPAL_PCI_SLOT_POWER_OFF, OPAL_SUCCESS);
+			break;
+		}
+
+		/* Power on */
+		if (slot->ops.get_link_state(slot, &link) != OPAL_SUCCESS)
+			link = 0;
+		if (link) {
+			slot->ops.prepare_link_change(slot, true);
+			pci_scan_bus(phb, pd->secondary_bus,
+				     pd->subordinate_bus,
+				     &pd->children, pd, true);
+			pci_add_device_nodes(phb, &pd->children, dn,
+					     &phb->lstate, 0);
+			pci_slot_set_state(slot, PCI_SLOT_STATE_NORMAL);
+			opal_queue_msg(OPAL_MSG_ASYNC_COMP, NULL, NULL,
+				       slot->async_token, dn->phandle,
+				       OPAL_PCI_SLOT_POWER_ON, OPAL_SUCCESS);
+		} else if (slot->retries-- == 0) {
+			pci_slot_set_state(slot, PCI_SLOT_STATE_NORMAL);
+			opal_queue_msg(OPAL_MSG_ASYNC_COMP, NULL, NULL,
+				       slot->async_token, dn->phandle,
+				       OPAL_PCI_SLOT_POWER_ON, OPAL_BUSY);
+		} else {
+			schedule_timer(&slot->timer, msecs_to_tb(10));
+		}
+
+		break;
+	default:
+		prlog(PR_ERR, "PCI SLOT %016llx: Unexpected state 0x%08x\n",
+		      slot->id, slot->state);
+	}
+}
+
+static int64_t opal_pci_set_power_state(uint64_t async_token,
+					uint64_t id,
+					uint64_t data)
+{
+	struct pci_slot *slot = pci_slot_find(id);
+	struct phb *phb = slot ? slot->phb : NULL;
+	struct pci_device *pd = slot ? slot->pd : NULL;
+	uint8_t *state = (uint8_t *)data;
+	int64_t rc;
+
+	if (!slot || !phb)
+		return OPAL_PARAMETER;
+
+	phb_lock(phb);
+	switch (*state) {
+	case OPAL_PCI_SLOT_POWER_OFF:
+		if (!slot->ops.prepare_link_change ||
+		    !slot->ops.set_power_state)
+			return OPAL_UNSUPPORTED;
+
+		slot->async_token = async_token;
+		slot->ops.prepare_link_change(slot, false);
+		rc = slot->ops.set_power_state(slot, PCI_SLOT_POWER_OFF);
+		break;
+	case OPAL_PCI_SLOT_POWER_ON:
+		if (!slot->ops.set_power_state ||
+		    !slot->ops.get_link_state)
+			return OPAL_UNSUPPORTED;
+
+		slot->async_token = async_token;
+		rc = slot->ops.set_power_state(slot, PCI_SLOT_POWER_ON);
+		break;
+	case OPAL_PCI_SLOT_OFFLINE:
+		if (!pd)
+			return OPAL_PARAMETER;
+
+		pci_remove_bus(phb, &pd->children);
+		rc = OPAL_SUCCESS;
+		break;
+	case OPAL_PCI_SLOT_ONLINE:
+		if (!pd)
+			return OPAL_PARAMETER;
+		pci_scan_bus(phb, pd->secondary_bus, pd->subordinate_bus,
+			     &pd->children, pd, true);
+		pci_add_device_nodes(phb, &pd->children, pd->dn,
+				     &phb->lstate, 0);
+		rc = OPAL_SUCCESS;
+		break;
+	default:
+		rc = OPAL_PARAMETER;
+	}
+
+	phb_unlock(phb);
+	if (rc == OPAL_ASYNC_COMPLETION) {
+		slot->retries = 500;
+		init_timer(&slot->timer, set_power_timer, slot);
+		schedule_timer(&slot->timer, msecs_to_tb(10));
+	}
+
+	return rc;
+}
+opal_call(OPAL_PCI_SET_POWER_STATE, opal_pci_set_power_state, 3);
+
 static int64_t opal_pci_set_phb_tce_memory(uint64_t phb_id,
 					   uint64_t tce_mem_addr,
 					   uint64_t tce_mem_size)
@@ -619,10 +800,9 @@ static int64_t opal_pci_set_phb_tce_memory(uint64_t phb_id,
 		return OPAL_PARAMETER;
 	if (!phb->ops->set_phb_tce_memory)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->set_phb_tce_memory(phb, tce_mem_addr, tce_mem_size);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -639,10 +819,9 @@ static int64_t opal_pci_get_phb_diag_data(uint64_t phb_id,
 		return OPAL_PARAMETER;
 	if (!phb->ops->get_diag_data)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->get_diag_data(phb, diag_buffer, diag_buffer_len);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -659,10 +838,9 @@ static int64_t opal_pci_get_phb_diag_data2(uint64_t phb_id,
 		return OPAL_PARAMETER;
 	if (!phb->ops->get_diag_data2)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->get_diag_data2(phb, diag_buffer, diag_buffer_len);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -678,13 +856,12 @@ static int64_t opal_pci_next_error(uint64_t phb_id, uint64_t *first_frozen_pe,
 		return OPAL_PARAMETER;
 	if (!phb->ops->next_error)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 
 	opal_pci_eeh_clear_evt(phb_id);
 	rc = phb->ops->next_error(phb, first_frozen_pe, pci_error_type,
 				  severity);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -703,11 +880,10 @@ static int64_t opal_pci_eeh_freeze_status2(uint64_t phb_id, uint64_t pe_number,
 		return OPAL_PARAMETER;
 	if (!phb->ops->eeh_freeze_status)
 		return OPAL_UNSUPPORTED;
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->eeh_freeze_status(phb, pe_number, freeze_state,
 					 pci_error_type, severity, phb_status);
-	phb->ops->unlock(phb);
-	pci_put_phb(phb);
+	phb_unlock(phb);
 
 	return rc;
 }
@@ -723,9 +899,9 @@ static int64_t opal_pci_set_phb_capi_mode(uint64_t phb_id, uint64_t mode, uint64
 	if (!phb->ops->set_capi_mode)
 		return OPAL_UNSUPPORTED;
 
-	phb->ops->lock(phb);
+	phb_lock(phb);
 	rc = phb->ops->set_capi_mode(phb, mode, pe_number);
-	phb->ops->unlock(phb);
+	phb_unlock(phb);
 	return rc;
 }
 opal_call(OPAL_PCI_SET_PHB_CAPI_MODE, opal_pci_set_phb_capi_mode, 3);
