@@ -221,6 +221,24 @@ static void pr_log_daemon_init(void)
 	}
 }
 
+/**
+ * ABI check that we can't perform at build-time: we want to ensure that the
+ * layout of struct host_interfaces matches that defined in the thunk.
+ */
+static void check_abi(void)
+{
+	extern unsigned char __hinterface_start, __hinterface_pad,
+	       __hinterface_end;
+
+	/* ensure our struct size matches the thunk definition */
+	assert((&__hinterface_end - &__hinterface_start)
+			== sizeof(struct host_interfaces));
+
+	/* ensure the padding layout is as expected */
+	assert((void *)&__hinterface_start == (void *)&hinterface);
+	assert((void *)&__hinterface_pad == (void *)&hinterface.reserved);
+}
+
 /* HBRT init wrappers */
 extern struct runtime_interfaces *call_hbrt_init(struct host_interfaces *);
 
@@ -631,7 +649,7 @@ int hservice_memory_error(uint64_t i_start_addr, uint64_t i_endAddr,
 	return 0;
 }
 
-void hservices_init(struct opal_prd_ctx *ctx, void *code)
+int hservices_init(struct opal_prd_ctx *ctx, void *code)
 {
 	uint64_t *s, *d;
 	int i, sz;
@@ -644,12 +662,19 @@ void hservices_init(struct opal_prd_ctx *ctx, void *code)
 	hbrt_entry.addr = (void *)htobe64((unsigned long)code + 0x100);
 	hbrt_entry.toc = 0; /* No toc for init entry point */
 
-	if (memcmp(code, "HBRTVERS", 8) != 0)
+	if (memcmp(code, "HBRTVERS", 8) != 0) {
 		pr_log(LOG_ERR, "IMAGE: Bad signature for "
 				"ibm,hbrt-code-image! exiting");
+		return -1;
+	}
 
 	pr_debug("IMAGE: calling ibm,hbrt_init()");
 	hservice_runtime = call_hbrt_init(&hinterface);
+	if (!hservice_runtime) {
+		pr_log(LOG_ERR, "IMAGE: hbrt_init failed, exiting");
+		return -1;
+	}
+
 	pr_log(LOG_NOTICE, "IMAGE: hbrt_init complete, version %016lx",
 			hservice_runtime->interface_version);
 
@@ -659,6 +684,8 @@ void hservices_init(struct opal_prd_ctx *ctx, void *code)
 	/* Byte swap the function pointers */
 	for (i = 0; i < sz; i++)
 		d[i] = be64toh(s[i]);
+
+	return 0;
 }
 
 static void fixup_hinterface_table(void)
@@ -1141,6 +1168,8 @@ static int handle_msg_occ_error(struct opal_prd_ctx *ctx,
 
 	proc = be64toh(msg->occ_error.chip);
 
+	pr_debug("FW: firmware signaled OCC error for proc 0x%x", proc);
+
 	if (!hservice_runtime->process_occ_error) {
 		pr_log_nocall("process_occ_error");
 		return -1;
@@ -1157,6 +1186,8 @@ static int handle_msg_occ_reset(struct opal_prd_ctx *ctx,
 
 	proc = be64toh(msg->occ_reset.chip);
 
+	pr_debug("FW: firmware requested OCC reset for proc 0x%x", proc);
+
 	if (!hservice_runtime->process_occ_reset) {
 		pr_log_nocall("process_occ_reset");
 		return -1;
@@ -1537,6 +1568,8 @@ static int run_prd_daemon(struct opal_prd_ctx *ctx)
 	/* log to syslog */
 	pr_log_daemon_init();
 
+	pr_debug("CTRL: Starting PRD daemon\n");
+
 	ctx->fd = -1;
 	ctx->socket = -1;
 
@@ -1592,18 +1625,20 @@ static int run_prd_daemon(struct opal_prd_ctx *ctx)
 
 	fixup_hinterface_table();
 
-	if (ctx->pnor.path) {
-		rc = pnor_init(&ctx->pnor);
-		if (rc) {
-			pr_log(LOG_ERR, "PNOR: Failed to open pnor: %m");
-			goto out_close;
-		}
+	rc = pnor_init(&ctx->pnor);
+	if (rc) {
+		pr_log(LOG_ERR, "PNOR: Failed to open pnor: %m");
+		goto out_close;
 	}
 
 	ipmi_init(ctx);
 
 	pr_debug("HBRT: calling hservices_init");
-	hservices_init(ctx, ctx->code_addr);
+	rc = hservices_init(ctx, ctx->code_addr);
+	if (rc) {
+		pr_log(LOG_ERR, "HBRT: Can't initialise HBRT");
+		goto out_close;
+	}
 	pr_debug("HBRT: hservices_init done");
 
 	/* Test a scom */
@@ -1619,6 +1654,7 @@ static int run_prd_daemon(struct opal_prd_ctx *ctx)
 	rc = 0;
 
 out_close:
+	pr_debug("CTRL: stopping PRD daemon\n");
 	pnor_close(&ctx->pnor);
 	if (ctx->fd != -1)
 		close(ctx->fd);
@@ -1942,6 +1978,8 @@ int main(int argc, char *argv[])
 	enum action action;
 	int rc;
 
+	check_abi();
+
 	ctx = &_ctx;
 	memset(ctx, 0, sizeof(*ctx));
 	ctx->vlog = pr_log_stdio;
