@@ -28,8 +28,6 @@
 #include <opal-msg.h>
 #include <errorlog.h>
 #include <centaur.h>
-/* XXX SRC's will be moved to errorlog.h and then remove fsp-elog.h */
-#include <fsp-elog.h>
 
 DEFINE_LOG_ENTRY(OPAL_RC_I2C_INIT, OPAL_PLATFORM_ERR_EVT, OPAL_I2C,
 		 OPAL_IO_SUBSYSTEM, OPAL_PREDICTIVE_ERR_DEGRADED_PERF,
@@ -140,13 +138,17 @@ DEFINE_LOG_ENTRY(OPAL_RC_I2C_RESET, OPAL_INPUT_OUTPUT_ERR_EVT, OPAL_I2C,
 #define I2C_STAT_SDA_INPUT_LEVEL	PPC_BIT(21)
 #define I2C_STAT_PORT_BUSY		PPC_BIT(22)
 #define I2C_STAT_INTERFACE_BUSY         PPC_BIT(23)
-#define I2C_STAT_FIFO_ENTRY_COUNT	  PPC_BITMASK(24, 31)
+#define I2C_STAT_FIFO_ENTRY_COUNT	PPC_BITMASK(24, 31)
 
 #define I2C_STAT_ANY_ERR (I2C_STAT_INVALID_CMD | I2C_STAT_LBUS_PARITY_ERR | \
 			  I2C_STAT_BKEND_OVERRUN_ERR | \
 			  I2C_STAT_BKEND_ACCESS_ERR | I2C_STAT_ARBT_LOST_ERR | \
 			  I2C_STAT_NACK_RCVD_ERR | I2C_STAT_STOP_ERR)
 
+/* Pseudo-status used for timeouts */
+#define I2C_STAT_PSEUDO_TIMEOUT		PPC_BIT(63)
+
+
 /* I2C extended status register */
 #define I2C_EXTD_STAT_REG		0xc
 #define I2C_EXTD_STAT_FIFO_SIZE		PPC_BITMASK(0, 7)
@@ -384,10 +386,15 @@ static int p8_i2c_prog_mode(struct p8_i2c_master_port *port, bool enhanced_mode)
 static void p8_i2c_complete_request(struct p8_i2c_master *master,
 				    struct i2c_request *req, int ret)
 {
+	struct p8_i2c_request *request =
+		container_of(req, struct p8_i2c_request, req);
+
 	/* We only complete the current top level request */
 	assert(req == list_top(&master->req_list, struct i2c_request, link));
 
 	cancel_timer_async(&master->timeout);
+	request->timeout = 0ul;
+
 	list_del(&req->link);
 	master->state = state_idle;
 	req->result = ret;
@@ -452,6 +459,8 @@ static void p8_i2c_translate_error(struct i2c_request *req, uint64_t status)
 		req->result = OPAL_I2C_ARBT_LOST;
 	else if (status & I2C_STAT_STOP_ERR)
 		req->result = OPAL_I2C_STOP_ERR;
+	else if (status & I2C_STAT_PSEUDO_TIMEOUT)
+		req->result = OPAL_I2C_TIMEOUT;
 }
 
 static void p8_i2c_status_error(struct p8_i2c_master_port *port,
@@ -461,10 +470,11 @@ static void p8_i2c_status_error(struct p8_i2c_master_port *port,
 	struct p8_i2c_master *master = port->master;
 	int rc;
 
-	/* Display any error other than I2C_INTR_NACK_RCVD_ERR since
-	 * getting NACK's is normal if Linux is probing the bus
+	/* Display any error other than I2C_INTR_NACK_RCVD_ERR or
+	 * timeout since getting NACK's is normal if Linux is probing
+	 * the bus and timeouts will have already logged something.
 	 */
-	if (!(status & I2C_STAT_NACK_RCVD_ERR)) {
+	if (!(status & (I2C_STAT_NACK_RCVD_ERR | I2C_STAT_PSEUDO_TIMEOUT))) {
 		log_simple_error(&e_info(OPAL_RC_I2C_TRANSFER),
 				 "I2C: Transfer error occurred\n");
 		p8_i2c_print_debug_info(port, req);
@@ -582,10 +592,15 @@ static void p8_i2c_status_data_request(struct p8_i2c_master *master,
 			rc = p8_i2c_fifo_write(master, master->obuf,
 					       req->offset_bytes);
 		}
+
+		/* For read, wait address phase to complete */
+		if (rc || req->op != SMBUS_WRITE)
+			break;
+
 		/* For writes, transition to data phase now */
-		if (rc == 0 && req->op == SMBUS_WRITE)
-			master->state = state_data;
-		break;
+		master->state = state_data;
+		fifo_free -= req->offset_bytes;
+		/* Fall through */
 	case state_data:
 		/* Sanity check */
 		if (master->bytes_sent >= req->rw_len) {
@@ -761,9 +776,12 @@ static void  p8_i2c_check_status(struct p8_i2c_master *master)
 	/* Handle the status in that order: errors, data requests and
 	 * command completion.
 	 */
-	if (status & I2C_STAT_ANY_ERR)
-		p8_i2c_status_error(port, req, status);
-	else if (status & I2C_STAT_DATA_REQ)
+	if (status & I2C_STAT_ANY_ERR) {
+		/* Mask status to avoid some unrelated bit overwriting
+		 * our pseudo-status "timeout" bit 63
+		 */
+		p8_i2c_status_error(port, req, status & I2C_STAT_ANY_ERR);
+	} else if (status & I2C_STAT_DATA_REQ)
 		p8_i2c_status_data_request(master, req, status);
 	else if (status & I2C_STAT_CMD_COMP)
 		p8_i2c_status_cmd_completion(master, req);
@@ -960,8 +978,12 @@ static int p8_i2c_start_request(struct p8_i2c_master *master,
 	now = schedule_timer(&master->poller, master->poll_interval);
 
 	/* Calculate and start timeout */
-	tbytes = req->rw_len + req->offset_bytes + 2;
-	request->timeout = now + tbytes * master->byte_timeout;
+	if (request->timeout) {
+		request->timeout += now;
+	} else {
+		tbytes = req->rw_len + req->offset_bytes + 2;
+		request->timeout = now + tbytes * master->byte_timeout;
+	}
 
 	/* Start the timeout */
 	schedule_timer_at(&master->timeout, request->timeout);
@@ -1033,9 +1055,19 @@ static void p8_i2c_free_request(struct i2c_request *req)
 	free(request);
 }
 
+static void p8_i2c_set_request_timeout(struct i2c_request *req,
+				       uint64_t duration)
+{
+	struct p8_i2c_request *request =
+		container_of(req, struct p8_i2c_request, req);
+
+	request->timeout = msecs_to_tb(duration);
+}
+
 static inline uint32_t p8_i2c_get_bit_rate_divisor(uint32_t lb_freq,
 						   uint32_t bus_speed)
 {
+	assert(bus_speed > 0);
 	return (((lb_freq / bus_speed) - 1) / 4);
 }
 
@@ -1043,6 +1075,8 @@ static inline uint64_t p8_i2c_get_poll_interval(uint32_t bus_speed)
 {
 	uint64_t usec;
 
+	assert(bus_speed > 0);
+
 	/* Polling Interval = 8 * (1/bus_speed) * (1/10) -> convert to uSec */
 	usec = ((8 * USEC_PER_SEC) / (10 * bus_speed));
 	return usecs_to_tb(usec);
@@ -1077,6 +1111,8 @@ static void p8_i2c_timeout(struct timer *t __unused, void *data, uint64_t now)
 		DBG("I2C: Timeout with request not expired\n");
 		goto exit;
 	}
+
+	request->timeout = 0ul;
 	port = container_of(req->bus, struct p8_i2c_master_port, bus);
 
 	/* Allright, we have a request and it has timed out ... */
@@ -1084,12 +1120,8 @@ static void p8_i2c_timeout(struct timer *t __unused, void *data, uint64_t now)
 			 "I2C: Request timeout !\n");
 	p8_i2c_print_debug_info(port, req);
 
-	/* Reset the engine */
-	p8_i2c_engine_reset(port);
-
-	/* Should we send a stop ? For now just complete */
-	p8_i2c_complete_request(master, req, OPAL_I2C_TIMEOUT);
-
+	/* Use the standard error path */
+	p8_i2c_status_error(port, req, I2C_STAT_PSEUDO_TIMEOUT);
  exit:
 	unlock(&master->lock);
 }
@@ -1360,6 +1392,7 @@ static void p8_i2c_init_one(struct dt_node *i2cm, enum p8_i2c_master_type type)
 		port->bus.queue_req = p8_i2c_queue_request;
 		port->bus.alloc_req = p8_i2c_alloc_request;
 		port->bus.free_req = p8_i2c_free_request;
+		port->bus.set_req_timeout = p8_i2c_set_request_timeout;
 		i2c_add_bus(&port->bus);
 
 		/* Add OPAL properties to the bus node */
