@@ -31,9 +31,18 @@ static const struct flash_info flash_info[] = {
 	{ 0xc22019, 0x02000000, FL_ERASE_ALL | FL_CAN_4B, "Macronix MXxxL25635F"},
 	{ 0xc2201a, 0x04000000, FL_ERASE_ALL | FL_CAN_4B, "Macronix MXxxL51235F"},
 	{ 0xef4018, 0x01000000, FL_ERASE_ALL,             "Winbond W25Q128BV"   },
+	{ 0xef4019, 0x02000000, FL_ERASE_ALL | FL_ERASE_64K | FL_CAN_4B |
+				FL_ERASE_BULK,
+							"Winbond W25Q256BV"},
 	{ 0x20ba20, 0x04000000, FL_ERASE_4K  | FL_ERASE_64K | FL_CAN_4B |
                                 FL_ERASE_BULK | FL_MICRON_BUGS,
                                                           "Micron N25Qx512Ax"   },
+	{ 0x20ba19, 0x02000000, FL_ERASE_4K  | FL_ERASE_64K | FL_CAN_4B |
+                                FL_ERASE_BULK | FL_MICRON_BUGS,
+                                                          "Micron N25Q256Ax"    },
+	{ 0x1940ef, 0x02000000, FL_ERASE_4K  | FL_ERASE_64K | FL_CAN_4B |
+				FL_ERASE_BULK | FL_MICRON_BUGS,
+							"Micron N25Qx256Ax"   },
 	{ 0x4d5444, 0x02000000, FL_ERASE_ALL | FL_CAN_4B, "File Abstraction"},
 	{ 0x55aa55, 0x00100000, FL_ERASE_ALL | FL_CAN_4B, "TEST_FLASH" },
 	{ 0xaa55aa, 0x02000000, FL_ERASE_ALL | FL_CAN_4B, "EMULATED_FLASH"},
@@ -749,8 +758,7 @@ static int flash_configure(struct flash_chip *c)
 			FL_DBG("LIBFLASH: Enabling controller 4B mode...\n");
 			rc = ct->set_4b(ct, true);
 			if (rc) {
-				FL_ERR("LIBFLASH: Failed"
-				       " to set controller 4b mode\n");
+				FL_ERR("LIBFLASH: Failed to set controller 4b mode\n");
 				return rc;
 			}
 		}
@@ -833,6 +841,10 @@ bail:
 		return rc;
 	}
 
+	/* The flash backend doesn't support reiniting it */
+	c->bl.keep_alive = true;
+	c->bl.reacquire = NULL;
+	c->bl.release = NULL;
 	c->bl.read = &flash_read;
 	c->bl.write = &flash_smart_write;
 	c->bl.erase = &flash_erase;
